@@ -4,7 +4,7 @@
  * This file is part of msmtp, an SMTP client.
  *
  * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008, 2010, 2011, 2012,
- * 2014, 2015, 2016, 2018, 2019, 2020
+ * 2014, 2015, 2016, 2018, 2019, 2020, 2021
  * Martin Lambers <marlam@marlam.de>
  * Martin Stenberg <martin@gnutiken.se> (passwordeval support)
  * Scott Shumate <sshumate@austin.rr.com> (aliases support)
@@ -34,6 +34,9 @@
 #include <strings.h>
 #include <ctype.h>
 #include <errno.h>
+#ifdef HAVE_FNMATCH_H
+# include <fnmatch.h>
+#endif
 
 #include "gettext.h"
 #define _(string) gettext(string)
@@ -305,6 +308,17 @@ account_t *find_account_by_envelope_from(list_t *acc_list, const char *from)
                 break;
             }
         }
+#ifdef HAVE_FNMATCH_H
+        else if (strchr(acc_from, '?') || strchr(acc_from, '*') || strchr(acc_from, '['))
+        {
+            /* This is a wildcard pattern according to glob(7) */
+            if (fnmatch(acc_from, from, 0) != FNM_NOMATCH)
+            {
+                a = acc_list->data;
+                break;
+            }
+        }
+#endif
         else
         {
             /* simple matching */
@@ -899,19 +913,17 @@ int get_password_eval(const char *arg, char **buf, char **errstr)
 
 
 /*
- * expand_from()
- *
- * see conf.h
+ * helper function for expand_from() and expand_domain()
  */
 
-int expand_from(char **from, char **errstr)
+static int expand_from_or_domain(char **str, int expand_U, char **errstr)
 {
     char* M = NULL;
     char* U = NULL;
     char* H = NULL;
     char* C = NULL;
 
-    if (strstr(*from, "%M"))
+    if (strstr(*str, "%M"))
     {
         char *sysconfdir;
         char *filename;
@@ -938,6 +950,10 @@ int expand_from(char **from, char **errstr)
         }
         fclose(f);
         buflen = strlen(buf);
+        if (buflen > 0 && buf[buflen - 1] == '\n')
+        {
+            buf[--buflen] = '\0';
+        }
         if (buflen > 0 && buf[buflen - 1] == '\r')
         {
             buf[--buflen] = '\0';
@@ -952,39 +968,39 @@ int expand_from(char **from, char **errstr)
         M = xstrdup(buf);
         sanitize_string(M);
     }
-    if (strstr(*from, "%U"))
+    if (expand_U && strstr(*str, "%U"))
     {
         U = get_username();
         sanitize_string(U);
     }
-    if (strstr(*from, "%H") || strstr(*from, "%C"))
+    if (strstr(*str, "%H") || strstr(*str, "%C"))
     {
         H = get_hostname();
         sanitize_string(H);
     }
-    if (strstr(*from, "%C"))
+    if (strstr(*str, "%C"))
     {
         C = net_get_canonical_hostname(H);
     }
 
     if (M)
     {
-        *from = string_replace(*from, "%M", M);
+        *str = string_replace(*str, "%M", M);
         free(M);
     }
     if (U)
     {
-        *from = string_replace(*from, "%U", U);
+        *str = string_replace(*str, "%U", U);
         free(U);
     }
     if (H)
     {
-        *from = string_replace(*from, "%H", H);
+        *str = string_replace(*str, "%H", H);
         free(H);
     }
     if (C)
     {
-        *from = string_replace(*from, "%C", C);
+        *str = string_replace(*str, "%C", C);
         free(C);
     }
 
@@ -992,6 +1008,30 @@ int expand_from(char **from, char **errstr)
 }
 
 
+/*
+ * expand_from()
+ *
+ * see conf.h
+ */
+
+int expand_from(char **from, char **errstr)
+{
+    return expand_from_or_domain(from, 1, errstr);
+}
+
+
+/*
+ * expand_domain()
+ *
+ * see conf.h
+ */
+
+int expand_domain(char **domain, char **errstr)
+{
+    return expand_from_or_domain(domain, 0, errstr);
+}
+
+
 /*
  * some small helper functions
  */
